﻿using ELTE.TravelAgency.Data;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;

namespace ELTE.TravelAgency.Admin.Persistence
{

    public enum Method { GET, POST, PUT, DELETE }
    /// <summary>
    /// Szolgáltatás alapú perzisztencia.
    /// </summary>
    public class TravelAgencyServicePersistence : ITravelAgencyPersistence
    {
        private HttpClient _client;
        private string _accessToken;
        private string _refreshToken;

        /// <summary>
        /// Szolgáltatás alkapú perszisztencia példányosítása.
        /// </summary>
        /// <param name="baseAddress">A szolgáltatás címe.</param>
        public TravelAgencyServicePersistence(String baseAddress)
        {
            _client = new HttpClient(); // a szolgáltatás kliense
            _client.BaseAddress = new Uri(baseAddress); // megadjuk neki a címet
        }

        /// <summary>
        /// Épületek betöltése.
        /// </summary>
        public async Task<IEnumerable<BuildingDTO>> ReadBuildingsAsync()
        {
            try
            {
                // a kéréseket a kliensen keresztül végezzük
                HttpResponseMessage response = await SendRequest("api/buildings/", Method.GET);
                if (response.IsSuccessStatusCode) // amennyiben sikeres a művelet
                {
                    IEnumerable<BuildingDTO> buildings = await response.Content.ReadAsAsync<IEnumerable<BuildingDTO>>();
                    // a tartalmat JSON formátumból objektumokká alakítjuk

                    // képek lekérdezése:
                    foreach (BuildingDTO building in buildings)
                    {
                        response = await SendRequest("api/images/building/" + building.Id, Method.GET);
                        if (response.IsSuccessStatusCode)
                        {
                            building.Images = (await response.Content.ReadAsAsync<IEnumerable<ImageDTO>>()).ToList();
                        }
                    }

                    return buildings;
                }
                else
                {
                    throw new PersistenceUnavailableException("Service returned response: " + response.StatusCode);
                }
            }
            catch (Exception ex)
            {
                throw new PersistenceUnavailableException(ex);
            }

        }

        /// <summary>
        /// Városok betöltése.
        /// </summary>
        public async Task<IEnumerable<CityDTO>> ReadCitiesAsync()
        {
            try
            {
                HttpResponseMessage response = await SendRequest("api/cities/", Method.GET);
                if (response.IsSuccessStatusCode)
                {
                    return await response.Content.ReadAsAsync<IEnumerable<CityDTO>>();
                }
                else
                {
                    throw new PersistenceUnavailableException("Service returned response: " + response.StatusCode);
                }
            }
            catch (Exception ex)
            {
                throw new PersistenceUnavailableException(ex);
            }
        }

        /// <summary>
        /// Épület létrehozása.
        /// </summary>
        /// <param name="building">Épület.</param>
        public async Task<Boolean> CreateBuildingAsync(BuildingDTO building)
        {
            try
            {
                HttpResponseMessage response = await SendRequest("api/buildings/", Method.POST, building); // az értékeket azonnal JSON formátumra alakítjuk
                building.Id = (await response.Content.ReadAsAsync<BuildingDTO>()).Id; // a válaszüzenetben megkapjuk a végleges azonosítót
                return response.IsSuccessStatusCode;
            }
            catch (Exception ex)
            {
                throw new PersistenceUnavailableException(ex);
            }
        }

        /// <summary>
        /// Épület módosítása.
        /// </summary>
        /// <param name="building">Épület.</param>
        public async Task<Boolean> UpdateBuildingAsync(BuildingDTO building)
        {
            try
            {
                HttpResponseMessage response = await SendRequest("api/buildings/", Method.PUT, building);
                return response.IsSuccessStatusCode;
            }
            catch (Exception ex)
            {
                throw new PersistenceUnavailableException(ex);
            }
        }

        /// <summary>
        /// Épület törlése.
        /// </summary>
        /// <param name="building">Épület.</param>
        public async Task<Boolean> DeleteBuildingAsync(BuildingDTO building)
        {
            try
            {
                HttpResponseMessage response = await SendRequest("api/buildings/" + building.Id, Method.DELETE);
                return response.IsSuccessStatusCode;
            }
            catch (Exception ex)
            {
                throw new PersistenceUnavailableException(ex);
            }
        }

        /// <summary>
        /// Épületkép létrehozása.
        /// </summary>
        /// <param name="image">Épületkép.</param>
        public async Task<Boolean> CreateBuildingImageAsync(ImageDTO image)
        {
            try
            {
                HttpResponseMessage response = await SendRequest("api/images/", Method.POST, image); // elküldjük a képet
                if (response.IsSuccessStatusCode)
                {
                    image.Id = await response.Content.ReadAsAsync<Int32>(); // a válaszüzenetben megkapjuk az azonosítót
                }
                return response.IsSuccessStatusCode;
            }
            catch (Exception ex)
            {
                throw new PersistenceUnavailableException(ex);
            }
        }

        /// <summary>
        /// Épületkép törlése.
        /// </summary>
        /// <param name="image">Épületkép.</param>
        public async Task<Boolean> DeleteBuildingImageAsync(ImageDTO image)
        {
            try
            {
                HttpResponseMessage response = await SendRequest("api/images/" + image.Id, Method.DELETE);
                return response.IsSuccessStatusCode;
            }
            catch (Exception ex)
            {
                throw new PersistenceUnavailableException(ex);
            }
        }


        /// <summary>
        /// Bejelentkezés.
        /// </summary>
        /// <param name="userName">Felhasználónév.</param>
        /// <param name="userPassword">Jelszó.</param>
        public async Task<Boolean> LoginAsync(String userName, String userPassword)
        {
            try
            {
                HttpResponseMessage response = await Send("api/account/login", Method.POST, new LoginDTO
                {
                    UserName = userName,
                    UserPassword = userPassword
                });

                try
                {
                    _accessToken = response.Headers.GetValues("Bearer").FirstOrDefault();
                    _refreshToken = response.Headers.GetValues("Refresh").FirstOrDefault();
                }
                catch (InvalidOperationException) // Bearer vagy Refresh nem létezik
                {
                    _accessToken = _refreshToken = null;
                }

                return response.IsSuccessStatusCode; // a művelet eredménye megadja a bejelentkezés sikerességét
            }
            catch (Exception ex)
            {
                throw new PersistenceUnavailableException(ex);
            }
        }

        /// <summary>
        /// Kijelentkezés.
        /// </summary>
        public async Task<Boolean> LogoutAsync()
        {
            try
            {
                HttpResponseMessage response = await SendRequest("api/account/logout", Method.GET);
                return !response.IsSuccessStatusCode;
            }
            catch (Exception ex)
            {
                throw new PersistenceUnavailableException(ex);
            }
        }

        private async Task<HttpResponseMessage> SendRequest(string url, Method method, object request = null)
        {

            var response = await Send(url, method, request);

            if (response.StatusCode == System.Net.HttpStatusCode.Unauthorized)
            {
                response = await Send("api/account/token", Method.POST, _refreshToken);
                _accessToken = response.Headers.GetValues("Bearer").FirstOrDefault();
                _refreshToken = response.Headers.GetValues("Refresh").FirstOrDefault();

                response = await Send(url, method, request);
            }

            return response;
        }

        private async Task<HttpResponseMessage> Send(string url, Method method, object request)
        {
            SetJWT();

            switch (method)
            {
                case Method.GET:
                    return await _client.GetAsync(url);
                case Method.POST:
                    return await _client.PostAsJsonAsync(url, request);
                case Method.PUT:
                    return await _client.PutAsJsonAsync(url, request);
                case Method.DELETE:
                    return await _client.DeleteAsync(url);
                default:
                    return null;
            }
        }

        private void SetJWT()
        {
            _client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", _accessToken);
        }

    }
}
