﻿using ELTE.TravelAgency.Data;
using ELTE.TravelAgency.Service.Models;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Linq;
using System.Threading.Tasks;

namespace ELTE.TravelAgency.Service.Controllers
{
	/// <summary>
	/// Felhasználókezelést biztosító vezérlő.
	/// </summary>
	[Route("api/[controller]")]
    [ApiController]
    [ApiConventionType(typeof(DefaultApiConventions))]
    public class AccountController : Controller
    {
	    /// <summary>
	    /// Authentikációs szolgáltatás.
	    /// </summary>
	    private readonly SignInManager<Guest> _signInManager;
        private readonly UserManager<Guest> _userManager;
        private readonly TravelAgencyContext _context;


        /// <summary>
        /// JWT authentikációt segítő osztály.
        /// </summary>
        private readonly IJwtService _jwtService;

		/// <summary>
		/// Vezérlő példányosítása.
		/// </summary>
		public AccountController(SignInManager<Guest> signInManager, UserManager<Guest> userManager, TravelAgencyContext context, IJwtService jwtService)
        {
	        _signInManager = signInManager;
            _userManager = userManager;
            _jwtService = jwtService;
            _context = context;
		}

        /// <summary>
        /// Bejelentkezés.
        /// </summary>
        /// <param name="loginDTO">Bejelentkezési adatok.</param>
        [HttpPost("login")]
        public async Task<IActionResult> Login([FromBody] LoginDTO loginDTO)
        {
            try
            {
	            // bejelentkeztetjük a felhasználót
	            var result = await _signInManager.PasswordSignInAsync(loginDTO.UserName, loginDTO.UserPassword, false, false);
	            if (!result.Succeeded) // ha nem sikerült, akkor nincs bejelentkeztetés
		            return Forbid();

                // authorizációs tokenek beállítása
                var user = _userManager.FindByNameAsync(loginDTO.UserName).Result;
                SetAuthorizationTokens(user);

                // ha sikeres volt az ellenőrzés
                return Ok();
            }
            catch
            {
				// Internal Server Error
	            return StatusCode(StatusCodes.Status500InternalServerError);
			}
        }

       

        /// <summary>
        /// Kijelentkezés.
        /// </summary>
        [HttpGet("logout")]
        [Authorize] // csak bejelentkezett felhasználóknak
        public async Task<IActionResult> Logout()
        {
            try
            {
				// kijelentkeztetjük az aktuális felhasználót
				await _signInManager.SignOutAsync();
				return Ok();
            }
            catch
            {
				// Internal Server Error
	            return StatusCode(StatusCodes.Status500InternalServerError);
			}
        }

        /// <summary>
        /// Kijelentkezés.
        /// </summary>
        [HttpPost("token")]
        public IActionResult RefreshTokens([FromBody] string token)
        {
            try
            {
                // kijelentkeztetjük az aktuális felhasználót
                var user = _context.Users.FirstOrDefault(u => u.RefreshToken == token);
                if(user == null)
                    return StatusCode(StatusCodes.Status403Forbidden);

                SetAuthorizationTokens(user);

                return Ok();
            }
            catch
            {
                // Internal Server Error
                return StatusCode(StatusCodes.Status500InternalServerError);
            }
        }

        private void SetAuthorizationTokens(Guest user)
        {
            // JWT token generálása
            string bearer = _jwtService.GenerateJWTToken(user);

            // kiállított JWT token beállítása a válasz HTTP headerben
            Response.Headers.Add("Bearer", bearer);
            Response.Headers.Add("Refresh", user.RefreshToken);
        }
    }
}
