using System;
using System.IO;
using System.Reflection;
using System.Text;
using ELTE.TravelAgency.Service;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Identity;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using ELTE.TravelAgency.Service.Models;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.IdentityModel.Tokens;
using Microsoft.OpenApi.Models;

var builder = WebApplication.CreateBuilder(args);

// Add services to the container.
builder.Services.AddDbContext<TravelAgencyContext>(options => // Dependency injection belltsa az adatbzis kontextushoz
    options.UseSqlServer(builder.Configuration.GetConnectionString("DefaultConnection")));

// Dependency injection belltsa az authentikcihoz
builder.Services.AddIdentity<Guest, IdentityRole<int>>(options =>
    {
        // Jelsz komplexitsra vonatkoz konfigurci
        options.Password.RequireDigit = true;
        options.Password.RequiredLength = 8;
        options.Password.RequireNonAlphanumeric = false;
        options.Password.RequireUppercase = true;
        options.Password.RequireLowercase = false;
        options.Password.RequiredUniqueChars = 3;

        // Hibs bejelentkezs esetn az (ideiglenes) kizrsra vonatkoz konfigurci
        options.Lockout.DefaultLockoutTimeSpan = TimeSpan.FromMinutes(30);
        options.Lockout.MaxFailedAccessAttempts = 10;
        options.Lockout.AllowedForNewUsers = true;

        // Felhasznlkezelsre vonatkoz konfigurci
        options.User.RequireUniqueEmail = true;
    })
    .AddEntityFrameworkStores<TravelAgencyContext>() // EF hasznlata a TravelAgencyContext entits kontextussal
    .AddDefaultTokenProviders(); // Alaprtelmezett token genertor hasznlata (pl. SecurityStamp-hez)

// Dependency injection belltsa a JWT konfigurcihoz
builder.Services.Configure<JwtConfig>(builder.Configuration.GetSection("Jwt"));
builder.Services.AddTransient<IJwtService, JwtService>();

// MVC
builder.Services.AddControllers();

// JWT authentikci
builder.Services.AddAuthentication(cfg =>
    {
        cfg.DefaultAuthenticateScheme = JwtBearerDefaults.AuthenticationScheme;
        cfg.DefaultChallengeScheme = JwtBearerDefaults.AuthenticationScheme;
    })
    .AddJwtBearer(cfg =>
    {
        cfg.TokenValidationParameters = new TokenValidationParameters
        {
            ValidIssuer = builder.Configuration["Jwt:JwtIssuer"],
            ValidAudience = builder.Configuration["Jwt:JwtIssuer"],
            IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(builder.Configuration["Jwt:JwtKey"]))
        };
    });

// Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
builder.Services.AddEndpointsApiExplorer();
// Swagger generator regisztrlsa
builder.Services.AddSwaggerGen(c =>
{
    // (nv, OpenApiInfo) prok megadsa szksges
    c.SwaggerDoc("v1", new OpenApiInfo { Title = "Travel Agency API", Version = "v1" });

    // XML API dokumentci felhasznlsa a Swaggerben
    var xmlFile = $"{Assembly.GetExecutingAssembly().GetName().Name}.xml";
    var xmlPath = Path.Combine(AppContext.BaseDirectory, xmlFile);
    c.IncludeXmlComments(xmlPath);
});


var app = builder.Build();

// Configure the HTTP request pipeline.
if (app.Environment.IsDevelopment())
{
    // Swagger hasznlata (JSON vgpontok)
    app.UseSwagger();

    // Swagger UI engedlyezse (bngszhet HTML vgpontok)
    app.UseSwaggerUI(c =>
    {
        // a JSON vgpont megadsa (s engedlyezse szksges)
        c.SwaggerEndpoint("/swagger/v1/swagger.json", "Travel Agency API V1");
    });
}

app.UseHttpsRedirection();

app.UseAuthentication();

app.UseAuthorization();

app.MapControllers();

using (var serviceScope = app.Services.CreateScope())
{
    // Adatbzis inicializlsa
    DbInitializer.Initialize(
        serviceScope.ServiceProvider,
        builder.Configuration.GetValue<string>("ImageStore"));
}

app.Run();